//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <stdio.h>
#include "ArmyLevelDialog.h"
#include "defs.h"

ArmyLevelDialog::ArmyLevelDialog(Army* army, PG_Widget* parent, PG_Rect rect)
    :PG_Window(parent, rect, "Your unit advanced a level!", WF_MODAL), d_army(army)
{
    // here the army can gain level 
    army->gainLevel();

    char buffer[31];
    buffer[30] = '\0';

    //I assume a size of 400*300 pixels here
    
    d_l_info = new PG_Label(this, PG_Rect(15, 200, 240, 20),"Choose a characteristic to raise.");
    d_b_ok = new PG_Button(this, 1, PG_Rect(260, 190, 130, 30),
                            "OK");
    d_b_ok->SetEventObject(MSG_BUTTONCLICK, this,
                        (MSG_CALLBACK_OBJ)&ArmyLevelDialog::b_ok_clicked);

    //the army pic
    d_b_pic = new PG_Button(this, 2, PG_Rect(15, 40, 50, 50), "");
    d_b_pic->EnableReceiver(false);
    d_b_pic->SetIcon(d_army->getPixmap(), 0, 0);

    //the name of the army
    d_l_name = new PG_Label(this, PG_Rect(15, 100, 200, 20), d_army->getName().c_str());

    snprintf(buffer, 30, "xp: %.2f", d_army->getXP());
    d_l_xp = new PG_Label(this, PG_Rect(15, 130, 95, 20), buffer);
    snprintf(buffer, 30, "%i", d_army->getLevel());
    d_l_level1 = new PG_Label(this, PG_Rect(15, 150, 45, 20),"level: ");
    d_l_level2 = new PG_Label(this, PG_Rect(60, 150, 30, 20), buffer);
    d_l_level2->SetFontColor(50, 255, 50);

    d_choose_def = new PG_RadioButton(this, 5, PG_Rect(220, 60, 40, 20)," +");
    d_choose_def->SetFontColor(50, 255, 50);
    d_choose_vit = new PG_RadioButton(this, 6, PG_Rect(220, 80, 40, 20),"", d_choose_def);
    d_choose_hp  = new PG_RadioButton(this, 7, PG_Rect(220, 100, 40, 20),"", d_choose_def);
    d_choose_str = new PG_RadioButton(this, 7, PG_Rect(220, 120, 40, 20),"", d_choose_def);
    d_choose_mov = new PG_RadioButton(this, 7, PG_Rect(220, 140, 40, 20),"", d_choose_def);
    d_choose_def->SetPressed();
                            
    //the head of the table for the single values
    d_l_stats = new PG_Label(this, PG_Rect(140, 40, 45, 20), "Stats");
    d_l_basic = new PG_Label(this, PG_Rect(280, 40, 45, 20), "Basic");
    d_l_real =  new PG_Label(this, PG_Rect(340, 40, 45, 20), "Real");
    d_l_stats->SetFontColor(0, 0, 0);
    d_l_basic->SetFontColor(0, 0, 0);
    d_l_real-> SetFontColor(0, 0, 0);

    //and the values themselves
    d_l_defensen = new PG_Label(this, PG_Rect(140, 60, 95, 20), "defense");
    snprintf(buffer, 30, "%i", d_army->getDefense());
    d_l_defenseb = new PG_Label(this, PG_Rect(280, 60, 45, 20), buffer);
    d_l_defenser = new PG_Label(this, PG_Rect(340, 60, 45, 20), buffer);

    d_l_vitalityn = new PG_Label(this, PG_Rect(140, 80, 96, 20), "vitality");
    snprintf(buffer, 30, "%i", d_army->getVitality());
    d_l_vitalityb = new PG_Label(this, PG_Rect(280, 80, 45, 20), buffer);
    d_l_vitalityr = new PG_Label(this, PG_Rect(340, 80, 45, 20), buffer);

    d_l_hpn = new PG_Label(this, PG_Rect(140, 100, 95, 20), "hp");
    snprintf(buffer, 30, "%i", d_army->getMaxHp());
    d_l_hpb = new PG_Label(this, PG_Rect(280, 100, 45, 20), buffer);
    snprintf(buffer, 30, "%i", d_army->getHP());
    d_l_hpr = new PG_Label(this, PG_Rect(340, 100, 45, 20), buffer);
    if (d_army->getHP() < d_army->getMaxHp())
        d_l_hpr->SetFontColor(200, 30, 30);

    d_l_strengthn = new PG_Label(this, PG_Rect(140, 120, 95, 20), "strength");
    snprintf(buffer, 30, "%i", d_army->getMaxStrength());
    d_l_strengthb = new PG_Label(this, PG_Rect(280, 120, 45, 20), buffer);
    snprintf(buffer, 30, "%i", d_army->getStrength());
    d_l_strengthr = new PG_Label(this, PG_Rect(340, 120, 45, 20), buffer);
    if (d_army->getMaxStrength() < d_army->getStrength())
        d_l_strengthr->SetFontColor(50, 255, 50);   //green highlight
    if (d_army->getMaxStrength() > d_army->getStrength())
        d_l_strengthr->SetFontColor(100, 30, 30);   //red highlight

    d_l_movesn = new PG_Label(this, PG_Rect(140, 140, 95, 20), "moves");
    snprintf(buffer, 30, "%i", d_army->getMaxMoves());
    d_l_movesb = new PG_Label(this, PG_Rect(280, 140, 45, 20), buffer);
    snprintf(buffer, 30, "%i", d_army->getMoves());
    d_l_movesr = new PG_Label(this, PG_Rect(340, 140, 45, 20), buffer);
    if (d_army->getMoves() < d_army->getMaxMoves())
        d_l_movesr->SetFontColor(200, 30, 20);

    //    d_l_movebonusn = new PG_Label(this, PG_Rect(200, 230, 100, 20), "move bonus");
    // if (d_l_armybonusv) d_l_armybonusv->SetFontColor(200, 30, 30);
    d_choose_def->SetEventObject(MSG_BUTTONCLICK, this,
      (MSG_CALLBACK_OBJ)&ArmyLevelDialog::stat_chosen);
    d_choose_vit->SetEventObject(MSG_BUTTONCLICK, this,
      (MSG_CALLBACK_OBJ)&ArmyLevelDialog::stat_chosen);
    d_choose_hp->SetEventObject(MSG_BUTTONCLICK, this,
      (MSG_CALLBACK_OBJ)&ArmyLevelDialog::stat_chosen);
    d_choose_str->SetEventObject(MSG_BUTTONCLICK, this,
      (MSG_CALLBACK_OBJ)&ArmyLevelDialog::stat_chosen);
    d_choose_mov->SetEventObject(MSG_BUTTONCLICK, this,
      (MSG_CALLBACK_OBJ)&ArmyLevelDialog::stat_chosen);

}

ArmyLevelDialog::~ArmyLevelDialog()
{
    delete d_b_ok;
    delete d_b_pic;
    delete d_l_info;
    delete d_l_name;
    delete d_l_stats;
    delete d_l_basic;
    delete d_l_real;
    delete d_l_strengthn;
    delete d_l_strengthb;
    delete d_l_strengthr;
    delete d_l_defensen;
    delete d_l_defenseb;
    delete d_l_defenser;
    delete d_l_vitalityn;
    delete d_l_vitalityb;
    delete d_l_vitalityr;
    delete d_l_hpn;
    delete d_l_hpb;
    delete d_l_hpr;
    delete d_l_movesn;
    delete d_l_movesb;
    delete d_l_movesr;
    delete d_l_xp;
    delete d_l_level1;
    delete d_l_level2;

    delete d_choose_def;
    delete d_choose_vit;
    delete d_choose_str;
    delete d_choose_hp;
    delete d_choose_mov;
}

PARAGUI_CALLBACK(ArmyLevelDialog::stat_chosen)
{

    d_choose_def->SetText("");
    d_choose_vit->SetText("");
    d_choose_str->SetText("");
    d_choose_hp-> SetText("");
    d_choose_mov->SetText("");

    (static_cast<PG_RadioButton*>(widget))->SetFontColor(50, 255, 50);
    (static_cast<PG_RadioButton*>(widget))->SetText(" +");
    return true;
}

PARAGUI_CALLBACK(ArmyLevelDialog::b_ok_clicked)
{


    if (d_choose_def->GetPressed())
    {
        d_army->setDefense((d_army->getDefense()+1));
    }    
    
    if (d_choose_vit->GetPressed())
    {
        d_army->setVitality((d_army->getVitality()+1));
    }    
    
    if (d_choose_str->GetPressed())
    {
        d_army->setMaxStrength((d_army->getMaxStrength()+1));
        d_army->setStrength(d_army->getMaxStrength());
    }    
	
    if (d_choose_hp->GetPressed()) 
    {
        d_army->setMaxHp((d_army->getMaxHp()+(int)(d_army->getMaxHp()/5)));
    }    
    
    if (d_choose_mov->GetPressed())
    {
        d_army->setMaxMoves((d_army->getMaxMoves()+(int)(d_army->getMaxMoves()/5)));
    }    

    // Here we must recalculate the Xpvalue because the unit has a new level and is stronger
    // Here is a simple one.
    d_army->setXpReward(d_army->getXpReward()+1);
    d_army->printDebugInfo();

    SendMessage(this, MSG_MODALQUIT, 0, 0);
    return true;
}

