//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "File.h"
#include <pgfilearchive.h>
#include <SDL_image.h>
#include "Configuration.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

list<string> File::s_armySetList;
list<string> File::s_tileSetList;

SDL_Surface* File::loadImage(string filename, bool alpha, bool colorkey)
{
    SDL_Surface* tmp = IMG_Load(filename.c_str());
    if (!tmp)
    {
        cerr << "ERROR: Couldn't load image " << filename << endl;
        return 0;
    }
    SDL_Surface* convertedImage;
    // 1:1 copy
    if (alpha)
    {
        convertedImage = SDL_DisplayFormatAlpha(tmp);
    }
    // strip alpha channel if any
    else
    {
        convertedImage = SDL_DisplayFormat(tmp);
    }
    SDL_FreeSurface(tmp);
    // set "hot-pink" colorkey
    if (colorkey)
        SDL_SetColorKey(convertedImage, SDL_SRCCOLORKEY, SDL_MapRGB(convertedImage->format, 255, 0, 255));

    return convertedImage;
}

list<string> File::scanArmysets()
{
    // I really _dislike_ physfs
    string path = Configuration::s_dataPath + "/army/";
	PG_FileArchive::RemoveAllArchives();
	PG_FileArchive::AddArchive(path.c_str());
	char** filelist = PG_FileArchive::EnumerateFiles("");
	for (int i = 0; filelist[i] != NULL; ++i)
	{
        string entry(filelist[i]);
        string::size_type idx = entry.find(".xml");
        if (idx != string::npos)
        {
            entry.replace(idx, 4, "");  //substitute the ".xml" with ""
			s_armySetList.push_back(entry);
        }
	}

    if (s_armySetList.empty())
    {
        cerr << "Couldn't find a single armyset!" << endl;
        cerr << "Please check the path settings in /etc/freelordsrc or ~/.freelordsrc" << endl;
        cerr << "Exiting!" << endl;
        exit(-1);
    }

	return s_armySetList;
}

string File::getArmyset(string armysetname)
{
	return Configuration::s_dataPath + "/army/" + armysetname + ".xml";
}

SDL_Surface* File::getArmyPicture(string armysetname, bool alpha)
{
    return loadImage(Configuration::s_dataPath + "/army/" + armysetname + ".png", alpha, false);
}

SDL_Surface* File::getArmyMask(string armysetname)
{
    string filename = Configuration::s_dataPath + "/army/" + armysetname + "_mask.png";
    return IMG_Load(filename.c_str());
}

string File::getMapset(string mapsetname)
{
	return Configuration::s_dataPath + "/tilesets/" + mapsetname + "/" + mapsetname + ".xml";
}

SDL_Surface* File::getMapsetPicture(string mapsetname, string picname,
                                    bool alpha, bool colorkey)
{
	return loadImage(Configuration::s_dataPath + "/tilesets/" + mapsetname + "/" + picname,
                     alpha, colorkey);
}

SDL_Surface* File::getMapsetMask(string mapsetname, string picname)
{
    string filename = Configuration::s_dataPath + "/tilesets/" + mapsetname + "/" + picname;
    return IMG_Load(filename.c_str());
}

string File::getMiscFile(string filename)
{
	return Configuration::s_dataPath + "/" + filename;
}

SDL_Surface* File::getMiscPicture(string picname, bool alpha)
{
	return loadImage(Configuration::s_dataPath + "/various/" + picname, alpha);
}

string File::getSavePath()
{
    return Configuration::s_savePath + "/";
}

list<string> File::scanTilesets(PG_DropDown* drop_down)
{
    // I really _dislike_ physfs 
    // dunno really (Andrea)
    string path = Configuration::s_dataPath + "/tilesets/";
    PG_FileArchive::RemoveAllArchives();
    PG_FileArchive::AddArchive(path.c_str());
    char** filelist = PG_FileArchive::EnumerateFiles("");

    for (int i = 0; filelist[i] != NULL; ++i) 
    {
      if (PG_FileArchive::IsDirectory(filelist[i]) )
	PG_FileArchive::AddArchive((path+string(filelist[i])).c_str());
    }
    
    filelist = PG_FileArchive::EnumerateFiles("");

    for (int j = 0; filelist[j] != NULL; ++j)
    {
        string entry(filelist[j]);
	string::size_type idx = entry.find(".xml");
	if (idx != string::npos)
	{
	    entry.replace(idx, 4, "");  //substitute the ".xml" with ""
	    s_tileSetList.push_back(entry);
	    drop_down->AddItem(entry.c_str());
	    cerr << "Found  TileSet \"" << entry << "\"." << endl;
	}
    }  
    if (s_tileSetList.empty())
    {
        cerr << "Couldn't find a single TileSet!" << endl;
        cerr << "Please check the path settings in /etc/freelordsrc or ~/.freelordsrc" << endl;
        cerr << "Exiting!" << endl;
	exit(-1);
    }
    else 
    {
       drop_down->SetText(s_tileSetList.front().c_str());
    }
    
    PG_FileArchive::RemoveAllArchives();
    
    return s_tileSetList;
}

// End of file
