// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef GAMEMAP_H
#define GAMEMAP_H

#include "maptile.h"
#include "xmlhelper.h"
#include "MapGenerator.h"

// Description: GameMap represent a single map. In most cases this will be
// the map that is currently played, but it might be used to preview a map
// in a mapeditor too
//
// Notes: GameMap was prefered over Map, because of the potential confusion
// with the std::map from the STL
//
// Implementation: In a previous design this has been a global variable, now
// it's a singleton.

class GameMap
{
    public:
        // Singleton creators
        static GameMap* getInstance();
        static GameMap* getInstance(std::string TilesetName);
        static GameMap* getInstance(XML_Helper* helper,
                                    std::string TilesetName);
        static GameMap* getInstance(XML_Helper* helper);

        static void deleteInstance();
	
		// Set methods
        void setPlayers(int players){d_players = players;}
        void setName(std::string name){d_name = name;}
        void setInfo(std::string info){d_info = info;}
        static void setSize(int size){s_size = size;}

		// Get methods
        int getPlayers(){return d_players;}
        std::string getName(){return d_name;}
        std::string getInfo(){return d_info;}
        TileSet* getTileSet(){return s_tileSet;}
        static int getSize(){return s_size;}
        Maptile* getTile(int x, int y);

		// Methods
        bool fill(MapGenerator* generator);
		bool save(XML_Helper* helper);

	protected:
        // CREATORS
		GameMap(std::string TilesetName);
		GameMap(XML_Helper* helper);
		GameMap(XML_Helper* helper, std::string TilesetName);
		~GameMap();
		
	private:
		// Methods
		bool isUsed(int x, int y);

		// Data
        static GameMap* s_instance;
        static TileSet* s_tileSet;  //a hack due to a bug which would show only
                                    //black tiles after loading a tileset twice
        static int s_size;

		Maptile** d_map;
		std::string d_name;
		std::string d_info;
		int d_players;
};

#endif

// End of file
