//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "QKillHero.h"
#include "signals.h"
#include <iostream>
#include <sstream>
#include <assert.h>
#include "stacklist.h"
#include <algorithm>
//=======================================================================
/** \brief An auxiliary function, maybe some day it'll be moved
           to some more general place.
 */
static void getEnemyHeroes(std::vector<Hero*>& dst);


using namespace std;

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)
//=======================================================================
QuestKillHero::QuestKillHero(QuestsManager& mgr, 
                             int hero) : Quest(mgr, hero)
{
    sdyingArmy.connect( SigC::slot(*this, &QuestKillHero::dyingArmy));
    // now let's move to the next player and find his hero,
    // after finishing with the next player and having not
    // chosen a hero procede to the next player... and so on
    // until we make a decision

    // determine the 'other' heroes count:

    Hero *hunted = chooseToKill();

    // if there are no heroes belonging to other players, 
    // we exit the program
    assert(hunted);
    d_hero_to_kill = hunted->getId();
    _initDescription(d_description, hunted);
}
//=======================================================================
void QuestKillHero::_initDescription(std::string& d, Hero *hero)
{
	std::stringstream s;
	s << "Kill the hero " << hero->getId() << " , his name is " 
      << hero->getName();
    s << " (player " << hero->getOwner()->getName()  << ")";
    d = s.str();
}
//=======================================================================
bool QuestKillHero::isFeasible()
{
    std::vector<Hero*> heroes;
    getEnemyHeroes(heroes);
    return (heroes.empty() == 0);
}
//=======================================================================
QuestKillHero::QuestKillHero(QuestsManager& q_mgr, int heroId,
                             XML_Helper* helper) : Quest(q_mgr, heroId)
{
    sdyingArmy.connect( SigC::slot(*this, &QuestKillHero::dyingArmy));

    // read hunted hero id
    helper->getData(d_hero_to_kill, "to_kill");

    // find this hero object:
    debug("load: hero_to_kill = " << d_hero_to_kill);

    Hero *hero = getHeroById(d_hero_to_kill);

    assert(hero);
    // finish initialization:
    _initDescription(d_description, hero);
}
//=======================================================================
Hero* QuestKillHero::chooseToKill()
{
    std::vector<Hero*> heroes;
    getEnemyHeroes(heroes);
    if (heroes.empty())
        return NULL;
    // ok - pick up a hero:
    unsigned int size = heroes.size();
    unsigned int idx = (rand() % size);
    return heroes[idx];
}
//=======================================================================
void QuestKillHero::getProgress(std::string& dst)
{
    dst += "You're still searching for him...";

    // add info about the location of the hunted:
    Stack *stack=0;

    // this one was unused
    //Hero *hero = getHeroById(d_hero_to_kill, &stack);

    debug("Hunted hero stack is = " << stack);
	std::stringstream s;
    s << "Seen lately near (" << stack->getPos().x << ", "
      << stack->getPos().y << ")";
    dst += s.str();
}
//=======================================================================
void QuestKillHero::dyingArmy(Army *army, culpritsType culprits)
{
    if (pending == 0)
        return;
    std::cout << "army is dying: " << army << "\n";
    // is it a hero we were hunting for? */
    if (army->getId() == d_hero_to_kill)
    {
        culpritsType::iterator it;
        // ok... now check if he was killed by our hero */
        int ourHero = getHeroId();
        for (it = culprits.begin(); it != culprits.end(); it++)
        {
            if ((*it) == ourHero)
            {
                debug("CONGRATULATIONS: QUEST 'KILL HERO' IS COMPLETED!");
                questCompleted();                 
                break;
            }
        }
    }
}
//=======================================================================
void QuestKillHero::getSuccessMsg(std::queue<std::string>& msgs)
{
	std::stringstream s;
    s << "You slayed the wicked hero " << d_hero_to_kill;
    msgs.push(s.str());
    msgs.push("He deserved that!");
}
//=======================================================================
bool QuestKillHero::save(XML_Helper* helper)
{
    return helper->saveData("to_kill", d_hero_to_kill);
}
//=======================================================================
static void getEnemyHeroes(std::vector<Hero*>& dst)
{
    // loop through the player's unit
    // for every hero check any pending quests
    Playerlist* list = Playerlist::getInstance();
    Player *player  = Playerlist::getActiveplayer();

    // we start from the next player after us:
	Playerlist::iterator pit = find(list->begin(), list->end(), player);
    do
    {
        pit++;
        if (pit == list->end())
            pit=list->begin();
        if ((*pit) != player)
        {
            // get all heroes from all player's stacks
            Stacklist* stacklist = (*pit)->getStacklist();
            for (Stacklist::const_iterator it = stacklist->begin(); 
                it != stacklist->end(); it++)
            {
                for (Stack::iterator sit = (*it)->begin(); 
                    sit != (*it)->end(); sit++)
	            {
                    if ((*sit)->isHero()) 
                        dst.push_back(dynamic_cast<Hero*>(*sit));
                }
            }
        }
    }
    while ((*pit) != player);
}
