//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "action.h"
#include <stdlib.h>
#include <sstream>
#include "stack.h"
#include "army.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<< x << endl;}
//#define debug(x)

Action::Action()
{
}

Action::~Action()
{
}

Action* Action::handle_package(data_package* mydata)
{
	Action* a = 0;
	switch (mydata->action)
	{
		case ACTION_STACK_FIGHT:
			a = new Action_Fight(mydata);
			break;

		case ACTION_STACK_SPLIT:
			a = new Action_Split(mydata);
			break;

		case ACTION_STACK_FIGHT_CITY:
			a = new Action_Fight_City(mydata);
			break;

		case ACTION_STACK_JOIN:
			a = new Action_Join(mydata);
			break;

		case ACTION_STACK_MOVED_ONE_STEP:
			a = new Action_Move_Step(mydata);
			break;

		case ACTION_CITY_OCCUPY:
			a = new Action_Occupy(mydata);
			break;

		case ACTION_CITY_RAZE:
			a = new Action_Raze(mydata);
			break;

		case ACTION_STACK_SEARCH_RUIN:
			a = new Action_Search(mydata);
			break;

		case ACTION_STACK_VISIT_TEMPLE:
			a =new Action_Temple(mydata);
			break;

		case ACTION_REWARD:
			a = new Action_Reward(mydata);
			break;

		case ACTION_CITY_UPGRADE_DEFENSE:
			a = new Action_Upgrade_Defense(mydata);
			break;

		case ACTION_CITY_BUY_PRODUCTION:
			a = new Action_Buy_Production(mydata);
			break;

		case ACTION_CITY_CHANGE_PRODUCTION:
			a = new Action_Change_Production(mydata);
			break;
	}
	return a;
}

Action* Action::handle_load(XML_Helper* helper)
{
	std::string s;
	helper->getData(s, "type");

	if (s == "ACTION_STACK_FIGHT")
		return (new Action_Fight(helper));

	if (s == "ACTION_STACK_SPLIT")
		return (new Action_Split(helper));

	if (s == "ACTION_STACK_FIGHT_CITY")
		return (new Action_Fight_City(helper));

	if(s == "ACTION_STACK_JOIN")
		return(new Action_Join(helper));

	if(s == "ACTION_STACK_MOVED_ONE_STEP")
		return (new Action_Move_Step(helper));

	if(s == "ACTION_CITY_OCCUPY")
		return (new Action_Occupy(helper));

	if(s == "ACTION_CITY_RAZE")
		return (new Action_Raze(helper));

    if (s == "ACTION_CITY_PILLAGE")
        return (new Action_Pillage(helper));

	if(s == "ACTION_STACK_SEARCH_RUIN")
		return (new Action_Search(helper));

	if(s == "ACTION_STACK_VISIT_TEMPLE")
		return (new Action_Temple(helper));

	if(s == "ACTION_REWARD")
		return (new Action_Reward(helper));

	if(s == "ACTION_CITY_UPGRADE_DEFENSE")
		return (new Action_Upgrade_Defense(helper));

	if(s == "ACTION_CITY_BUY_PRODUCTION")
		return (new Action_Buy_Production(helper));

	if(s == "ACTION_CITY_CHANGE_PRODUCTION")
		return (new Action_Change_Production(helper));
	
	return 0;
}

//-----------------------------------------------------------------------------
//Action_Fight

Action_Fight::Action_Fight()
	:Action()
{
	d_id = ACTION_STACK_FIGHT;

	for (int i =0; i < 8; i++)
		d_attacker_damages[i] = d_defender_damages[i] = 0;
}

Action_Fight::Action_Fight(data_package* mydata)
{
	std::list<int>::iterator it = mydata->data.begin();

	d_attacker_id = (unsigned int) (*it);	it++;
	d_defender_id = (unsigned int) (*it);	it++;

	for (int i = 0; i < 8; i++, it++)
		d_attacker_damages[i] = (*it);
	
	for (int i = 0; i < 8; i++, it++)
		d_defender_damages[i] = (*it);
}

Action_Fight::Action_Fight(XML_Helper* helper)
{
	d_id = ACTION_STACK_FIGHT;

	helper->getData(d_attacker_id, "attacker");
	helper->getData(d_defender_id, "defender");

	std::string s;
	std::istringstream si;
	
	//first attacker damages
	helper->getData(s, "a_damage");
	si.str(s);
	for (int i = 0; i < 8; i++)
		si >> d_attacker_damages[i];
	
	helper->getData(s, "d_damage");
	si.str(s);
	for (int i = 0; i < 8; i++)
		si >> d_defender_damages[i];
}

Action_Fight::~Action_Fight()
{
}

std::string Action_Fight::dump()
{
	std::stringstream s;

	s <<"Attacker " <<d_attacker_id <<"fought Defender ";
	s <<d_defender_id <<std::endl;

	s <<"attacker damage:";
	for (int i = 0; i < 8; i++)
		s <<" " <<d_attacker_damages[i];
	
	s <<"\ndefender damage:";
	for (int i = 0; i < 8; i++)
		s <<" " <<d_defender_damages[i];
	s <<std::endl;

	return s.str();
}

bool Action_Fight::save(XML_Helper* helper)
{
	bool retval = true;
	std::stringstream sa, sd;

	for (int i = 0; i < 7; i++)
	{
		sa <<d_attacker_damages[i] <<" ";
		sd <<d_defender_damages[i] <<" ";
	}
	sa <<d_attacker_damages[7];
	sd <<d_defender_damages[7];

	retval &= helper->openTag("action");
	retval &= helper->saveData("type", std::string("ACTION_STACK_FIGHT"));
	retval &= helper->saveData("attacker", d_attacker_id);
	retval &= helper->saveData("defender", d_defender_id);
	retval &= helper->saveData("a_damage", sa.str());
	retval &= helper->saveData("d_damage", sd.str());
	retval &= helper->closeTag();

	return retval;
}

data_package Action_Fight::get_data()
{
	data_package mydata;
	mydata.action = ACTION_STACK_FIGHT;

	mydata.data.push_back((int)(d_attacker_id));
	mydata.data.push_back((int)(d_defender_id));

	for (int i = 0; i < 8; i++)
		mydata.data.push_back(d_attacker_damages[i]);
	
	for (int i = 0; i < 8; i++)
		mydata.data.push_back(d_defender_damages[i]);

	return mydata;
}

bool Action_Fight::fill_data(Stack* attacker, Stack* defender,
				Stack* orig_attacker, Stack* orig_defender)
{
	if ((attacker->size() != orig_attacker->size())
		|| (defender->size() != orig_defender->size())
		|| (attacker->size() > 8) || (attacker->empty())
		|| (defender->size() > 8) || (defender->empty()))
	{
		std::cerr <<"Action_Fight::fill_data(): wrong stack size\n";
		return false;
	}

	debug("Action_Fight::fill_data()")
	
	d_attacker_id = orig_attacker->getId();
	d_defender_id = orig_defender->getId();

	Stack::iterator it, orig_it;

	it = attacker->begin();
	orig_it = orig_attacker->begin();

	for (int i = 0; it != attacker->end(); i++, it++, orig_it++)
		d_attacker_damages[i] = (*orig_it)->getHP() - (*it)->getHP();	
	
	it = defender->begin();
	orig_it = orig_defender->begin();

	for (int i = 0; it != defender->end(); i++, it++, orig_it++)
		d_defender_damages[i] = (*orig_it)->getHP() - (*it)->getHP();
	
	return true;
}

//-----------------------------------------------------------------------------
//Action_Split

Action_Split::Action_Split()
	:Action()
{
	d_id = ACTION_STACK_SPLIT;
	
	for (int i = 0; i < 8; i++)
		d_armies_moved[i] = 0;
}

Action_Split::Action_Split(data_package* mydata)
{
	d_id = ACTION_STACK_SPLIT;

	std::list<int>::iterator it = mydata->data.begin();
	
	d_orig_stack_id = static_cast<unsigned int>(*it);		it++;
	d_added_stack_id = static_cast<unsigned int>(*it);		it++;

	for (int i = 0; i < 8; i++, it++)
		d_armies_moved[i] = static_cast<unsigned int>(*it);
}	

Action_Split::Action_Split(XML_Helper* helper)
{
	d_id = ACTION_STACK_SPLIT;

	helper->getData(d_orig_stack_id, "orig_army");
	helper->getData(d_added_stack_id, "new_army");

	std::string s;
	std::istringstream si;

	helper->getData(s, "moved");
	si.str(s);
	for (int i = 0; i < 8; i++)
		si >>d_armies_moved[i];
}

Action_Split::~Action_Split()
{
}

std::string Action_Split::dump()
{
	std::stringstream s;

	s <<"Stack " <<d_orig_stack_id <<" splitted with new stack ";
	s <<d_added_stack_id <<".\n";

	s <<"moved armies: ";
	for (int i = 0; i < 8; i++)
		s <<d_armies_moved[i] <<" ";
	s <<"\n";

	return s.str();
}

bool Action_Split::save(XML_Helper* helper)
{
	bool retval = true;
	std::stringstream s;

	for (int i = 0; i < 7; i++)
		s <<d_armies_moved[i] <<" ";
	s <<d_armies_moved[7];

	retval &= helper->openTag("action");
	retval &= helper->saveData("type", std::string("ACTION_STACK_SPLIT"));
	retval &= helper->saveData("orig_army", d_orig_stack_id);
	retval &= helper->saveData("new_army", d_added_stack_id);
	retval &= helper->saveData("moved", s.str());
	retval &= helper->closeTag();

	return retval;
}

data_package Action_Split::get_data()
{
	data_package mydata;

	mydata.action = ACTION_STACK_SPLIT;
	mydata.data.push_back(static_cast<int>(d_orig_stack_id));
	mydata.data.push_back(static_cast<int>(d_added_stack_id));

	for (int i = 0; i < 8; i++)
		mydata.data.push_back(static_cast<int>(d_armies_moved[i]));
	
	return mydata;
}

bool Action_Split::fill_data(Stack* orig_stack, Stack* added_stack)
{
	if ((orig_stack->size() > 8) ||(added_stack->size() > 8)
		|| (orig_stack->empty()) || (added_stack->empty()))
	{
		std::cerr <<"Action_Split::fill_data(): wrong stack size\n";
		return false;
	}
	
	debug("Action_Split::fill_data()")

	d_orig_stack_id = orig_stack->getId();
	d_added_stack_id = added_stack->getId();

	Stack::iterator it = added_stack->begin();
	for (int i = 0; it != added_stack->end(); it++, i++)
		d_armies_moved[i] = (*it)->getId();
	
	return true;
}

//-----------------------------------------------------------------------------
//Action_Join

Action_Join::Action_Join()
	:Action()
{
	d_id = ACTION_STACK_JOIN;
	d_orig_id = d_joining_id = 0;
}

Action_Join::Action_Join(data_package* mydata)
{
	d_id = ACTION_STACK_JOIN;
	d_orig_id = *(mydata->data.begin());
	d_joining_id = *((mydata->data.begin())++);
}

Action_Join::Action_Join(XML_Helper* helper)
{
	d_id = ACTION_STACK_JOIN;

	helper->getData(d_orig_id, "receiver");
	helper->getData(d_joining_id, "joining");
}

Action_Join::~Action_Join()
{
}

std::string Action_Join::dump()
{
	std::stringstream s;

	s <<"Stack " <<d_joining_id <<" joined stack " <<d_orig_id <<"\n";

	return s.str();
}

bool Action_Join::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("action");
	retval &= helper->saveData("type", std::string("ACTION_STACK_JOIN"));
	retval &= helper->saveData("receiver", d_orig_id);
	retval &= helper->saveData("joining", d_joining_id);
	retval &= helper->closeTag();

	return retval;
}

data_package Action_Join::get_data()
{
	data_package mydata;

	mydata.action = ACTION_STACK_JOIN;
	mydata.data.push_back(static_cast<unsigned int>(d_orig_id));
	mydata.data.push_back(static_cast<unsigned int>(d_joining_id));

	return mydata;
}

bool Action_Join::fill_data(Stack* orig, Stack* joining)
{
	if ((orig->empty()) || (joining->empty())
		|| (orig->size() + joining->size() > 8))
	{
		std::cerr <<"Action_Join::fill_data(): wrong stack size\n";
		return false;
	}
	
	debug("Action_Join::fill_data")
	
	d_orig_id = orig->getId();
	d_joining_id = joining->getId();
	return true;
}

//-----------------------------------------------------------------------------
//Action_Fight_City

Action_Fight_City::Action_Fight_City()
	:Action()
{
	d_id = ACTION_STACK_FIGHT_CITY;

	for (int i = 0; i < Fight::ATTACKER_MAX; i++)
	{
		d_attacker_ids[i] = 0;
		for (int k = 0; k < 8; k++)
			d_attacker_damages[i][k] = 0;
	}

	for (int i = 0; i < Fight::DEFENDER_MAX; i++)
	{
		d_defender_ids[i] = 0;
		for (int k = 0; k < 8; k++)
			d_defender_damages[i][k] = 0;
	}
}

Action_Fight_City::Action_Fight_City(data_package* mydata)
	:Action()
{
	d_id = ACTION_STACK_FIGHT_CITY;
	std::list<int>::iterator it = mydata->data.begin();

	for (int i = 0; i < Fight::ATTACKER_MAX; i++, it++)
		d_attacker_ids[i] = static_cast<unsigned int>(*it);
	
	for (int i = 0; i < Fight::DEFENDER_MAX; i++, it++)
		d_defender_ids[i] = static_cast<unsigned int>(*it);
	
	for (int i = 0; i < Fight::ATTACKER_MAX; i++)
		for (int k = 0; k < 8; k++, it++)
			d_attacker_damages[i][k] = (*it);
	
	for (int i = 0; i < Fight::DEFENDER_MAX; i++)
		for (int k = 0; k < 8;k++, it++)
			d_defender_damages[i][k] = (*it);
}

Action_Fight_City::Action_Fight_City(XML_Helper* helper)
{
	d_id = ACTION_STACK_FIGHT_CITY;

	std::string sa, sd, sad, sdd;
	std::istringstream ssa, ssd, ssad, ssdd;

	helper->getData(sa, "attackers");
	ssa.str(sa);
	helper->getData(sd, "defenders");
	ssd.str(sd);
	helper->getData(sad, "att_damage");
	ssad.str(sad);
	helper->getData(sdd, "def_damage");
	ssdd.str(sdd);
	
	for (int i = 0; i < Fight::ATTACKER_MAX; i++)
	{
		ssa >>d_attacker_ids[i];
		for (int j = 0; j < 8; j++)
			ssad >>d_attacker_damages[i][j];
	}

	for (int i = 0; i < Fight::DEFENDER_MAX; i++)
	{
		ssd >>d_defender_ids[i];
		for (int j = 0; j < 8; j++)
			ssdd >>d_defender_damages[i][j];
	}
}

Action_Fight_City::~Action_Fight_City()
{
}

std::string Action_Fight_City::dump()
{
	std::stringstream s;
	int a = 0;
	int d = 0;

	while ((a < Fight::ATTACKER_MAX) && (d_attacker_ids[a] != 0))
		a++;
	while ((d < Fight::DEFENDER_MAX) && (d_defender_ids[a] != 0))
		d++;

	s <<"attackers ";
	for (int i = 0; i < a; i++)
		s <<d_attacker_ids[i] <<" ";
	
	s <<"\n fought against city defenders ";
	for (int i = 0; i < d; i++)
		s <<d_defender_ids[i] <<" ";
	
	s <<"\ncausalties on attacker side:\n";
	for (int i = 0; i < a; i++)
	{
		for (int j = 0; j < 8; j++)
			s <<d_attacker_damages[i][j] <<" ";
		s <<std::endl;
	}

	s <<"defender causalties:\n";
	for (int i = 0; i< d; i++)
	{
		for (int j = 0; j < 8; j++)
			s <<d_defender_damages[i][j] <<" ";
		s <<std::endl;
	}

	return s.str();
}

bool Action_Fight_City::save(XML_Helper* helper)
{
	bool retval = true;
	std::stringstream sa, sd, da, dd;

	for (int i = 0; i < Fight::ATTACKER_MAX; i++)
	{
		sa <<d_attacker_ids[i] <<" ";
		for (int j = 0; j < 8; j++)
			da <<d_attacker_damages[i][j] <<" ";
	}

	for (int i = 0; i < Fight::DEFENDER_MAX; i++)
	{
		sd <<d_defender_ids[i] <<" ";
		for (int j = 0; j < 8; j++)
			dd <<d_defender_damages[i][j] <<" ";
	}

	retval &= helper->openTag("action");
	retval &= helper->saveData("type", std::string("ACTION_STACK_FIGHT_CITY"));
	retval &= helper->saveData("attackers", sa.str());
	retval &= helper->saveData("defenders", sd.str());
	retval &= helper->saveData("att_damage", da.str());
	retval &= helper->saveData("def_damage", dd.str());
	retval &= helper->closeTag();

	return retval;
}

data_package Action_Fight_City::get_data()
{
	data_package mydata;

	mydata.action = ACTION_STACK_FIGHT_CITY;
	
	for (int i = 0; i < Fight::ATTACKER_MAX; i++)
		mydata.data.push_back(static_cast<int>(d_attacker_ids[i]));
	
	for (int i = 0; i < Fight::DEFENDER_MAX; i++)
		mydata.data.push_back(static_cast<int>(d_defender_ids[i]));
	
	for (int i = 0; i < Fight::ATTACKER_MAX; i++)
		for (int k = 0; k < 8; k++)
			mydata.data.push_back(d_attacker_damages[i][k]);
	
	for (int i = 0; i < Fight::DEFENDER_MAX; i++)
		for (int k = 0; k < 8; k++)
			mydata.data.push_back(d_defender_damages[i][k]);
	
	return mydata;
}

bool Action_Fight_City::fill_data(std::vector<Stack*> orig_attackers,
					std::vector<Stack*> orig_defenders,
					std::vector<Stack*> attackers,
					std::vector<Stack*> defenders)
{
	if ((orig_attackers.size() > (unsigned int)Fight::ATTACKER_MAX)
		|| (orig_attackers.size() != attackers.size())
		|| (orig_defenders.size() > (unsigned int)Fight::DEFENDER_MAX)
		|| (orig_defenders.size() != defenders.size()))
	{
		std::cerr <<"Action_Fight_City::fill_data: wrong vector size\n";
		return false;
	}

	debug("Action_Fight_City::fill_data()")

	std::vector<Stack*>::iterator it, it2;
	Stack::iterator sit, sit2;

	//calculate attacker damages etc.
	it = orig_attackers.begin();
	it2 = attackers.begin();
	for (int i = 0;	it != orig_attackers.end(); it++, it2++, i++)
	{
		d_attacker_ids[i] = (*it)->getId();
		sit = (*it)->begin();
		sit2 = (*it2)->begin();
		for (int k = 0; sit != (*it)->end(); sit++, sit2++, k++)
			d_attacker_damages[i][k] = (*sit)->getHP() - (*sit2)->getHP();
	}

	//now the same for the defenders
	it = orig_defenders.begin();
	it2 = defenders.begin();
	for (int i = 0;	it != orig_defenders.end(); it++, it2++, i++)
	{
		d_defender_ids[i] = (*it)->getId();
		sit = (*it)->begin();
		sit2 = (*it2)->begin();
		for (int k = 0; sit != (*it)->end(); sit++, sit2++, k++)
			d_defender_damages[i][k] = (*sit)->getHP() - (*sit2)->getHP();
	}

	return true;
}

//-----------------------------------------------------------------------------
//Action_Move_Step

Action_Move_Step::Action_Move_Step()
	:Action(), d_stack_id(0)
{
	d_id = ACTION_STACK_MOVED_ONE_STEP;
	d_dest.x = d_dest.y = 0;
}

Action_Move_Step::Action_Move_Step(data_package* mydata)
	:Action()
{
	d_id = ACTION_STACK_MOVED_ONE_STEP;
	std::list<int>::iterator it = mydata->data.begin();
	
	d_stack_id = static_cast<unsigned int>(*it);	it++;
	d_dest.x = (*it);				it++;
	d_dest.y = (*it);				it++;
}

Action_Move_Step::Action_Move_Step(XML_Helper* helper)
{
	d_id = ACTION_STACK_MOVED_ONE_STEP;

	helper->getData(d_stack_id, "stack");

	int i;
	helper->getData(i, "x");
	d_dest.x = i;
	helper->getData(i, "y");
	d_dest.y = i;
}

Action_Move_Step::~Action_Move_Step()
{
}

std::string Action_Move_Step::dump()
{
	std::stringstream s;

	s <<"Stack " <<d_stack_id <<" moved to (";
	s <<d_dest.x <<"," <<d_dest.y <<")\n";
	
	return s.str();
}

bool Action_Move_Step::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("action");
	retval &= helper->saveData("type", std::string("ACTION_STACK_MOVED_ONE_STEP"));
	retval &= helper->saveData("stack", d_stack_id);
	retval &= helper->saveData("x", d_dest.x);
	retval &= helper->saveData("y", d_dest.y);
	retval &= helper->closeTag();

	return retval;
}

data_package Action_Move_Step::get_data()
{
	data_package mydata;
	
	mydata.action = ACTION_STACK_MOVED_ONE_STEP;
	mydata.data.push_back(static_cast<int>(d_stack_id));
	mydata.data.push_back(d_dest.x);
	mydata.data.push_back(d_dest.y);

	return mydata;
}

bool Action_Move_Step::fill_data(Stack* s, PG_Point dest)
{
	d_stack_id = s->getId();
	d_dest = dest;
	return true;
}

//-----------------------------------------------------------------------------
//Action_Occupy

Action_Occupy::Action_Occupy()
	:Action(), d_city_id(0)
{
	d_id = ACTION_CITY_OCCUPY;
}

Action_Occupy::Action_Occupy(XML_Helper* helper)
{
	d_id = ACTION_CITY_OCCUPY;

	helper->getData(d_city_id, "city_id");
}

Action_Occupy::Action_Occupy(data_package* mydata)
{
	d_id = ACTION_CITY_OCCUPY;
	d_city_id = static_cast<unsigned int>(*(mydata->data.begin()));
}

Action_Occupy::~Action_Occupy()
{
}

std::string Action_Occupy::dump()
{
	std::stringstream s;

	s <<"City " <<d_city_id <<" occupied\n";

	return s.str();
}

bool Action_Occupy::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("action");
	retval &= helper->saveData("type", std::string("ACTION_CITY_OCCUPY"));
	retval &= helper->saveData("city_id", d_city_id);
	retval &= helper->closeTag();

	return retval;
}

data_package Action_Occupy::get_data()
{
	data_package mydata;
	mydata.action = ACTION_CITY_OCCUPY;
	mydata.data.push_back(static_cast<int>(d_city_id));
	return mydata;
}

bool Action_Occupy::fill_data(City* c)
{
	d_city_id = c->getId();
	return true;
}

//-----------------------------------------------------------------------------
//Action_Raze

Action_Raze::Action_Raze()
	:Action(), d_city_id(0)
{
	d_id = ACTION_CITY_RAZE;
}

Action_Raze::Action_Raze(data_package* mydata)
	:Action()
{
	d_id = ACTION_CITY_RAZE;
	d_city_id = static_cast<unsigned int>(*(mydata->data.begin()));
}

Action_Raze::Action_Raze(XML_Helper* helper)
{
	d_id = ACTION_CITY_RAZE;

	helper->getData(d_city_id, "city_id");
}

Action_Raze::~Action_Raze()
{
}

std::string Action_Raze::dump()
{
	std::stringstream s;

	s <<"City " <<d_city_id <<" razed.\n";

	return s.str();
}

bool Action_Raze::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("action");
	retval &= helper->saveData("type", std::string("ACTION_CITY_RAZE"));
	retval &= helper->saveData("city_id", d_city_id);
	retval &= helper->closeTag();

	return retval;
}

data_package Action_Raze::get_data()
{
	data_package mydata;
	mydata.action = ACTION_CITY_RAZE;
	mydata.data.push_back(static_cast<int>(d_city_id));
	return mydata;
}

bool Action_Raze::fill_data(City* c)
{
	d_city_id = c->getId();
	return true;
}

//-----------------------------------------------------------------------------
//Action_Pillage

Action_Pillage::Action_Pillage()
    :Action(), d_city_id(0)
{
    d_id = ACTION_CITY_PILLAGE;
}

Action_Pillage::Action_Pillage(XML_Helper* helper)
{
    d_id = ACTION_CITY_PILLAGE;
    helper->getData(d_city_id, "city");
}

Action_Pillage::~Action_Pillage()
{
}

std::string Action_Pillage::dump()
{
    std::stringstream s;
    s <<"city " <<d_city_id <<"pillaged.\n";
    return s.str();
}

bool Action_Pillage::save(XML_Helper* helper)
{
    bool retval = true;

    retval &= helper->openTag("action");
    retval &= helper->saveData("type", std::string("ACTION_CITY_PILLAGE"));
    retval &= helper->saveData("city_id", d_city_id);
    retval &= helper->closeTag();

    return retval;
}

bool Action_Pillage::fill_data(City* c)
{
    d_city_id = c->getId();
    return true;
}

//-----------------------------------------------------------------------------
//Action_Search

Action_Search::Action_Search()
	:Action(), d_ruin_id(0), d_stack_id(0), d_searched(0)
{
	d_id = ACTION_STACK_SEARCH_RUIN;
	for (int i = 0; i < 8; i++)
	{
		d_keeper_damages[i] = 0;
		d_seeker_damages[i] = 0;
	}
}

Action_Search::Action_Search(data_package* mydata)
{
	d_id = ACTION_STACK_SEARCH_RUIN;
	std::list<int>::iterator it = mydata->data.begin();
	
	d_ruin_id = static_cast<unsigned int>(*it);	it++;
	d_stack_id = static_cast<unsigned int>(*it);	it++;
	d_searched = static_cast<bool>(*it);		it++;

	for (int i = 0; i < 8; i++)
	{
		d_keeper_damages[i] = (*it);
		it++;
	}

	for (int i = 0; i < 8; i++)
	{
		d_seeker_damages[i] = (*it);
		it++;
	}
}

Action_Search::Action_Search(XML_Helper* helper)
{
	d_id = ACTION_STACK_SEARCH_RUIN;

	helper->getData(d_ruin_id, "ruin");
	helper->getData(d_stack_id, "seeker");
	helper->getData(d_searched, "searched");

	std::string ss, sk;
	std::istringstream sss, ssk;

	helper->getData(ss, "seeker_damage");
	helper->getData(sk, "keeper_damage");
	sss.str(ss);
	ssk.str(sk);
	
	for (int i = 0; i < 8; i++)
	{
		sss >>d_seeker_damages[i];
		ssk >>d_keeper_damages[i];
	}
}

Action_Search::~Action_Search()
{
}

std::string Action_Search::dump()
{
	std::stringstream s;

	s <<"Ruin " <<d_ruin_id <<" searched by stack " <<d_stack_id <<".\n";

	s <<"keeper damage: ";
	for (int i = 0; i < 8; i++)
		s <<d_keeper_damages[i] <<" ";

	s <<"\nexplorer damages: ";
	for (int i = 0; i < 8; i++)
		s <<d_seeker_damages[i] <<" ";
	
	s <<"Ruin has" <<(d_searched? " ":" not ") <<"been searched.\n";

	return s.str();
}

bool Action_Search::save(XML_Helper* helper)
{
	bool retval = true;
	std::stringstream sk, ss;

	for (int i = 0; i < 8; i++)
	{
		sk <<d_keeper_damages[i] <<" ";
		ss <<d_seeker_damages[i] <<" ";
	}

	retval &= helper->openTag("action");
	retval &= helper->saveData("type", std::string("ACTION_STACK_SEARCH_RUIN"));
	retval &= helper->saveData("ruin", d_ruin_id);
	retval &= helper->saveData("seeker", d_stack_id);
	retval &= helper->saveData("keeper_damage", sk.str());
	retval &= helper->saveData("seeker_damage", ss.str());
	retval &= helper->saveData("searched", d_searched);
	retval &= helper->closeTag();

	return retval;
}

data_package Action_Search::get_data()
{
	data_package mydata;
	mydata.action = ACTION_STACK_SEARCH_RUIN;
	mydata.data.push_back(static_cast<int>(d_ruin_id));
	mydata.data.push_back(static_cast<int>(d_stack_id));
	mydata.data.push_back(static_cast<int>(d_searched));
	for (int i = 0; i < 8; i++)
		mydata.data.push_back(d_keeper_damages[i]);
	for (int i = 0; i < 8; i++)
		mydata.data.push_back(d_seeker_damages[i]);
	
	return mydata;
}

bool Action_Search::fill_data(Ruin* r,Stack* orig_keepers, Stack*orig_explorers,
				Stack* explorers, bool searched)
{
	d_ruin_id = r->getId();
	d_stack_id = explorers->getId();
	d_searched = searched;

	//no fight, so we are finished here
	if ((orig_keepers == 0) || (explorers == 0) || (orig_explorers == 0))
		return true;

	Stack::iterator it, it2;
	it = r->getOccupant()->begin();
	it2 = orig_keepers->begin();

	for (int i = 0; it2 != orig_keepers->end(); it++, it2++, i++)
		d_keeper_damages[i] = (*it2)->getHP() - (*it)->getHP();

	it = explorers->begin();
	it2 = orig_explorers->begin();
	for (int i = 0; it2 != orig_explorers->end(); it++, it2++, i++)
		d_seeker_damages[i] = (*it2)->getHP() - (*it)->getHP();
	
	return true;
}

//-----------------------------------------------------------------------------
//Action_Temple

Action_Temple::Action_Temple()
	:Action(), d_temple_id(0), d_stack_id(0)
{
	d_id = ACTION_STACK_VISIT_TEMPLE;
}

Action_Temple::Action_Temple(data_package* mydata)
	:Action()
{
	d_id = ACTION_STACK_VISIT_TEMPLE;

	std::list<int>::iterator it = mydata->data.begin();

	d_temple_id = static_cast<unsigned int>(*it);	it++;
	d_stack_id = static_cast<unsigned int>(*it);
}

Action_Temple::Action_Temple(XML_Helper* helper)
{
	d_id = ACTION_STACK_VISIT_TEMPLE;

	helper->getData(d_temple_id, "temple");
	helper->getData(d_stack_id, "stack");
}

Action_Temple::~Action_Temple()
{
}

std::string Action_Temple::dump()
{
	std::stringstream s;

	s <<"Stack " <<d_stack_id <<"visited temple " <<d_temple_id <<".\n";

	return s.str();
}

bool Action_Temple::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("action");
	retval &= helper->saveData("type", std::string("ACTION_STACK_VISIT_TEMPLE"));
	retval &= helper->saveData("temple", d_temple_id);
	retval &= helper->saveData("stack", d_stack_id);
	retval &= helper->closeTag();

	return retval;
}

data_package Action_Temple::get_data()
{
	data_package mydata;
	
	mydata.action = ACTION_STACK_VISIT_TEMPLE;
	mydata.data.push_back(static_cast<int>(d_temple_id));
	mydata.data.push_back(static_cast<int>(d_stack_id));

	return mydata;
}

bool Action_Temple::fill_data(Temple* t, Stack* s)
{
	d_temple_id = t->getId();
	d_stack_id = s->getId();

	return true;
}

//-----------------------------------------------------------------------------
//Action_Upgrade_Defense

Action_Upgrade_Defense::Action_Upgrade_Defense()
	:Action(), d_city_id(0)
{
	d_id = ACTION_CITY_UPGRADE_DEFENSE;
}

Action_Upgrade_Defense::Action_Upgrade_Defense(data_package* mydata)
	:Action()
{
	d_id = ACTION_CITY_UPGRADE_DEFENSE;
	d_city_id = static_cast<unsigned int>(*(mydata->data.begin()));
}

Action_Upgrade_Defense::Action_Upgrade_Defense(XML_Helper* helper)
{
	d_id = ACTION_CITY_UPGRADE_DEFENSE;

	helper->getData(d_city_id, "city_id");
}

Action_Upgrade_Defense::~Action_Upgrade_Defense()
{
}

std::string Action_Upgrade_Defense::dump()
{
	std::stringstream s;

	s <<"Defense of city " <<d_city_id <<" upgraded.\n";

	return 	s.str();
}

bool Action_Upgrade_Defense::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("action");
	retval &= helper->saveData("type", std::string("ACTION_CITY_UPGRADE_DEFENSE"));
	retval &= helper->saveData("city_id", d_city_id);
	retval &= helper->closeTag();

	return retval;
}

data_package Action_Upgrade_Defense::get_data()
{
	data_package mydata;
	
	mydata.action = ACTION_CITY_UPGRADE_DEFENSE;
	mydata.data.push_back(static_cast<int>(d_city_id));

	return mydata;
}

bool Action_Upgrade_Defense::fill_data(City* c)
{
	d_city_id = c->getId();
	return true;
}

//-----------------------------------------------------------------------------
//Action_Buy_Production

Action_Buy_Production::Action_Buy_Production()
	:Action(), d_city_id(0), d_prod(0)
{
	d_id = ACTION_CITY_BUY_PRODUCTION;
}

Action_Buy_Production::Action_Buy_Production(data_package* mydata)
	:Action()
{
	d_id = ACTION_CITY_BUY_PRODUCTION;
	std::list<int>::iterator it = mydata->data.begin();

	d_city_id = static_cast<unsigned int>(*it);	it++;
	d_prod = (*it);
}

Action_Buy_Production::Action_Buy_Production(XML_Helper* helper)
{
	d_id = ACTION_CITY_BUY_PRODUCTION;

	helper->getData(d_city_id, "city_id");
	helper->getData(d_prod, "production");
}

Action_Buy_Production::~Action_Buy_Production()
{
}

std::string Action_Buy_Production::dump()
{
	std::stringstream s;

	s <<"Production " <<d_prod <<"bought in city " <<d_city_id <<".\n";

	return s.str();
}

bool Action_Buy_Production::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("action");
	retval &= helper->saveData("type", std::string("ACTION_CITY_BUY_PRODUCTION"));
	retval &= helper->saveData("city_id", d_city_id);
	retval &= helper->saveData("production", d_prod);
	retval &= helper->closeTag();

	return retval;
}

data_package Action_Buy_Production::get_data()
{
	data_package mydata;
	
	mydata.action = ACTION_CITY_BUY_PRODUCTION;
	mydata.data.push_back(static_cast<int>(d_city_id));
	mydata.data.push_back(d_prod);

	return mydata;
}

bool Action_Buy_Production::fill_data(City* c, int prod)
{
	d_city_id = c->getId();
	d_prod = prod;

	return true;
}

//-----------------------------------------------------------------------------
//Action_Change_Production

Action_Change_Production::Action_Change_Production()
	:Action(), d_city_id(0), d_prod(0)
{
	d_id = ACTION_CITY_CHANGE_PRODUCTION;
}

Action_Change_Production::Action_Change_Production(data_package* mydata)
{
	d_id = ACTION_CITY_CHANGE_PRODUCTION;

	std::list<int>::iterator it = mydata->data.begin();

	d_city_id = static_cast<unsigned int>(*it);	it++;
	d_prod = (*it);
}

Action_Change_Production::Action_Change_Production(XML_Helper* helper)
{
	d_id = ACTION_CITY_CHANGE_PRODUCTION;

	helper->getData(d_city_id, "city_id");
	helper->getData(d_prod, "production");
}

Action_Change_Production::~Action_Change_Production()
{
}

std::string Action_Change_Production::dump()
{
	std::stringstream s;

	s <<"Production in city " <<d_city_id <<" changed to " <<d_prod <<".\n";

	return s.str();
}

bool Action_Change_Production::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("action");
	retval &= helper->saveData("type", std::string("ACTION_CITY_CHANGE_PRODUCTION"));
	retval &= helper->saveData("city_id", d_city_id);
	retval &= helper->saveData("production", d_prod);
	retval &= helper->closeTag();

	return retval;
}

data_package Action_Change_Production::get_data()
{
	data_package mydata;

	mydata.action = ACTION_CITY_CHANGE_PRODUCTION;
	mydata.data.push_back(static_cast<int>(d_city_id));
	mydata.data.push_back(d_prod);

	return mydata;
}

bool Action_Change_Production::fill_data(City* c, int prod)
{
	d_city_id = c->getId();
	d_prod = prod;

	return true;
}

//-----------------------------------------------------------------------------
//Action_Reward

Action_Reward::Action_Reward()
	:Action(), d_stack_id(0)
{
	d_id = ACTION_REWARD;
	d_reward.type = 0;
}

Action_Reward::Action_Reward(data_package* mydata)
{
	d_id = ACTION_REWARD;
	std::list<int>::iterator it = mydata->data.begin();

	d_stack_id = static_cast<unsigned int>(*it);		it++;
	d_reward.type = static_cast<unsigned int>(*it);		it++;
	d_reward.subtype = static_cast<unsigned int>(*it);	it++;
	d_reward.amount = (*it);
}

Action_Reward::Action_Reward(XML_Helper* helper)
{
	d_id = ACTION_REWARD;

	helper->getData(d_reward.type, "reward_type");
	helper->getData(d_reward.subtype, "reward_sybtype");
	helper->getData(d_reward.amount, "reward_amount");
}

Action_Reward::~Action_Reward()
{
}

std::string Action_Reward::dump()
{
	std::stringstream s;

	s <<"Stack " <<d_stack_id <<" is given reward:\n";
	s <<"type=" <<d_reward.type <<" subtype=" <<d_reward.subtype;
	s <<" amount=" <<d_reward.amount <<"\n";

	return s.str();
}

bool Action_Reward::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("action");
	retval &= helper->saveData("type", std::string("ACTION_REWARD"));
	retval &= helper->saveData("reward_type", d_reward.type);
	retval &= helper->saveData("reward_subtype", d_reward.subtype);
	retval &= helper->saveData("reward_amount", d_reward.amount);
	retval &= helper->closeTag();

	return retval;
}

data_package Action_Reward::get_data()
{
	data_package mydata;
	mydata.action = ACTION_REWARD;

	mydata.data.push_back(static_cast<int>(d_stack_id));
	mydata.data.push_back(static_cast<int>(d_reward.type));
	mydata.data.push_back(static_cast<int>(d_reward.subtype));
	mydata.data.push_back(d_reward.amount);

	return mydata;
}

bool Action_Reward::fill_data(Stack* s, fl_Treasure reward)
{
	if (s)
    {
		d_stack_id = s->getId();
    }
	d_reward = reward;

	return true;
}
