//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "citylist.h"
#include "city.h"
#include "playerlist.h"

//#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
#define debug(x)

Citylist* Citylist::s_instance = 0;


Citylist* Citylist::getInstance()
{
    if (s_instance == 0)
        s_instance = new Citylist();

    return s_instance;
}

Citylist* Citylist::getInstance(XML_Helper* helper)
{
    if (s_instance)
        deleteInstance();

    s_instance = new Citylist(helper);
    return s_instance;
}

void Citylist::deleteInstance()
{
    if (s_instance)
        delete s_instance;

    s_instance = 0;
}

Citylist::Citylist()
{
}

Citylist::Citylist(XML_Helper* helper)
{
    //the only two entries are size and version which we ignore for now
    helper->registerTag("city", SigC::slot((*this), &Citylist::load));	
}

Citylist::~Citylist()
{
}

// Number of cities for specific player
int Citylist::countCities(Player* player) const
{
	int cities = 0;
	
	for (const_iterator it = begin(); it != end(); it++)
	{
		if ((*it).getPlayer() == player) cities++;
	}
	
	return cities;
}

void Citylist::nextTurn(Player* p)
{
	debug("next_turn(" <<p->getName() <<")");
	for (iterator it = begin(); it != end(); it++)
	{
        if ((*it).getPlayer() == p)
    		(*it).nextTurn();
	}
}

City* Citylist::getNearestEnemyCity(const PG_Point& pos) 
{
    int diff = 9999;
    Citylist::iterator diffit;
	for (Citylist::iterator it = begin(); it != end(); ++it)
	{
	    if ((*it).getPlayer() != Playerlist::getInstance()->getActiveplayer())
	    {
    	    PG_Point p = (*it).getPos();
    		int delta = abs(p.x - pos.x) + abs(p.y - pos.y);
    		
    		if (diff > delta)
            {
                diff = delta;
                diffit = it;
    		}
        }
	}
	return &(*diffit);
}

City* Citylist::getFirstCity(Player* p)
{
    for (iterator it = begin(); it != end(); it++)
        if ((*it).getPlayer() == p)
            return &(*it);

    return 0;
}

bool Citylist::save(XML_Helper* helper)
{
	bool retval = true;

	retval &= helper->openTag("citylist");
	retval &= helper->saveData("size", size());	//for checking later

	for (iterator it = begin(); it != end(); it++)
		it->save(helper);
	
	retval &= helper->closeTag();

	return retval;
}

bool Citylist::load(std::string tag, XML_Helper* helper)
{
	if (tag != "city")
		return false;
	
	City c(helper);
	push_back(c);

	return true;
}

// End of file
