//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FL_COUNTER_H
#define FL_COUNTER_H

#include "xmlhelper.h"
#include <string>
#include <sigc++/sigc++.h>

/* The purpose of this class is very simple. Each object (player etc.) has a
 * unique id by which it may be accessed (this isn't important for now, but
 * becomes crucial as soon as you play e.g. over the network). As of FL 0.2.2,
 * this is done by a private static member of each class which tells the current
 * counter. But now imagine that you load a game: which position is the counter?
 * To guarantee unique id's with this scheme is much hassle. So the solution is
 * to create a new classcounter which hands out unique id's and whose status
 * is saved together with a savegame. This has the further advantage that you
 * can guarantee unique id's for every object , not only for each of all
 * players, stacks etc. I don't know if this may be of use at some time
 * in the future, but it is a non-avoidable side-effect, so I'll regard it as
 * an advantage. :)
 */

class FL_Counter : public SigC::Object
{
    public:
        FL_Counter(unsigned int start = 1);
        FL_Counter(XML_Helper* helper);
        ~FL_Counter();

        unsigned int getNextId();

        bool save(XML_Helper* helper);

    private:
        unsigned int d_curID;
};

extern FL_Counter* fl_counter;
#endif
