//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#include <stdlib.h>
#include <fstream>

#include "hero.h"
#include "armyset.h"
#include "File.h"


std::vector<std::string> Hero::s_namelist;

Hero::Hero(Armyset* armyset, int type, std::string name, Player* owner)
    :Army(armyset, type, owner), d_name(name)
{
    if (s_namelist.empty())
        loadNames();

    if (name == "")  //set a random name
    {
        int entry = rand() % s_namelist.size();
        d_name = s_namelist[entry];
    }
}

Hero::Hero(Armyset* armyset, std::string name, Player* owner)
    :Army(armyset, rand()%2, owner), d_name(name)
{
    if (s_namelist.empty())
        loadNames();

    if (name == "")  //set a random name
    {
        int entry = rand() % s_namelist.size();
        d_name = s_namelist[entry];
    }
}

Hero::Hero(Armyset* armyset, int type, XML_Helper* helper)
    :Army(armyset, type, helper)
{
    helper->getData(d_name, "name");
}

Hero::Hero(Hero& h)
    :Army(h)
{
    d_name = h.d_name;
}

Hero::~Hero()
{
}

bool Hero::save(XML_Helper* helper)
{
    bool retval = true;

 	retval &= helper->openTag("hero");
	retval &= helper->saveData("id", d_id);
	retval &= helper->saveData("type", d_type);
	retval &= helper->saveData("hp", d_hp);
	retval &= helper->saveData("moves", d_moves);
	retval &= helper->saveData("strength", d_strength);
	retval &= helper->saveData("xp", d_xp);
	retval &= helper->saveData("expvalue", getXpReward());
	retval &= helper->saveData("level", d_level);
	retval &= helper->saveData("blessed", d_blessed);
        retval &= helper->saveData("name", d_name);
	retval &= helper->closeTag();

    return retval;
}

void Hero::loadNames()
{
    //Load hero names from a name file. This could certainly be improved
    //(e.g. separating male/female hero names), but should work well
    //enough for now.
    ifstream name_file(File::getMiscFile("heronames").c_str());
    if (name_file)
    {
        int entries;
        std::string curname;
        
        name_file >> entries;

        for (int i = 0; i < entries; i++)
        {
            name_file >> curname;
            s_namelist.push_back(curname);
        }

        name_file.close();
    }
}
