//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef PLAYERLIST_H
#define PLAYERLIST_H

#include <list>
#include <string>
#include <pgmessageobject.h>
#include "player.h"
#include <sigc++/object_slot.h>

class Playerlist : public std::list<Player*>, public SigC::Object
{
	public:

        //Singelton creators
        static Playerlist* getInstance();
        static Playerlist* getInstance(XML_Helper* helper);
        static void deleteInstance();
        //this one is a static function because it is needed very often and
        //calling "Playerlist::getActiveplayer()" is much shorter than
        //calling "Playerlist::getInstance()->getActiveplayer()"
		static Player* getActiveplayer() {return d_activeplayer;}
        
		// MANIPULATORS
		void nextPlayer();
		void checkPlayers();
        void playerStatusChanged();
		void setNeutral(Player* neutral) {d_neutral = neutral;}

		// ACCESSORS
		Player* getNeutral() const {return d_neutral;}
		Player* getPlayer(std::string name);
		Player* getPlayer(unsigned int id);
		int getPlayerNr(Player* p);
		
		//Load/Save
		bool save(XML_Helper* helper);
		bool load(std::string, XML_Helper* helper);

        SigC::Signal0<void> sgameWon;
        SigC::Signal1<void, Player*> splayerDead;
    
    protected:
		// CREATORS
		Playerlist();
		Playerlist(XML_Helper* helper);
		~Playerlist();
        
	private:
		iterator fl_erase(iterator it);

        //static pointers for the singleton
        static Playerlist* s_instance;
        
		// DATA
		static Player* d_activeplayer;
		Player* d_neutral;
};

#endif // PLAYERLIST_H
