//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "real_player.h"
#include <stdlib.h>
#include <algorithm>
#include "playerlist.h"
#include "armysetlist.h"
#include "FightDialog.h"
#include "stacklist.h"
#include "citylist.h"
#include <fstream>
#include "path.h"
#include "GameMap.h"
#include "army.h"
#include "hero.h"

#include <assert.h>
#include "signals.h"

using namespace std;

#define debug(x) {cerr<<__FILE__<<": "<<__LINE__<<": "<<x<<endl;}
//#define debug(x)

RealPlayer::RealPlayer(string name, string armyset, SDL_Color color)
    :Player(name, armyset, color, Player::HUMAN),d_attacker_xp(0),d_defender_xp(0)
{
}

RealPlayer::RealPlayer(string name, string armyset, SDL_Color color,
			Player::TYPE type)
    :Player(name, armyset, color, type),d_attacker_xp(0),d_defender_xp(0)
{
}

RealPlayer::RealPlayer(const Player& player)
	:Player(player),d_attacker_xp(0),d_defender_xp(0)
{
}

RealPlayer::RealPlayer(XML_Helper* helper)
	:Player(helper),d_attacker_xp(0),d_defender_xp(0)
{
}

RealPlayer::~RealPlayer()
{
}

bool RealPlayer::startTurn()
{
	clearActionlist();

	return true;
}

bool RealPlayer::invadeCity(City* c)
{
    //for the realplayer, this function doesn't do a lot.
    //However, an AI player has to decide here what to do (occupy, raze,
    //pillage)
    sinvadingCity.emit(c);
    return true;
}

bool RealPlayer::recruitHero(Hero* hero, int cost)
{
    //for the realplayer, this function also just raises a signal and looks
    //what to do next
    
    return srecruitingHero.emit(hero, cost);
}

bool RealPlayer::stackSplit(Stack* s)
{
	debug("RealPlayer::stackSplit()")

	Army* ungrouped = s->getFirstUngroupedArmy();
	if (!ungrouped)		//no armies to split
    {
		return false;
    }
	
	bool all_ungrouped = true;	//the whole army would be split
	for (Stack::iterator it = s->begin(); it != s->end(); it++)
    {
		if ((*it)->isGrouped())
        {
			all_ungrouped = false;
        }
    }
	if (all_ungrouped)
    {
		return false;
    }
	
	Stack* new_stack = new Stack(this, s->getPos());

	while (ungrouped)
	{
		new_stack->push_back(ungrouped);
		s->erase(find(s->begin(), s->end(), ungrouped));
		
		ungrouped->setGrouped(true);
		ungrouped = s->getFirstUngroupedArmy();
	}

	d_stacklist->push_back(new_stack);

	Action_Split* item = new Action_Split();
	item->fill_data(s, new_stack);
	d_actions.push_back(item);

	return true;
}

bool RealPlayer::stackJoin(Stack* receiver, Stack* joining)
{
	debug("RealPlayer::stackJoin()")

	if ((receiver == 0) || (joining == 0))
		return false;

	if ((receiver->size() + joining->size()) > 8)
		return false;
	
	for (Stack::iterator it = joining->begin(); it != joining->end(); it++)
	{
		receiver->push_back(*it);
		(*it)->setGrouped(true);
	}

	Action_Join* item = new Action_Join();
	item->fill_data(receiver, joining);
	d_actions.push_back(item);

	joining->clear();	//clear only erases the pointers not the armies
	d_stacklist->flRemove(joining);
    supdatingStack.emit(0);
	
	return true;
}

bool RealPlayer::stackMove(Stack* s)
{
    debug("player::stackMove(Stack*)")
    
	if (s->getPath()->empty())
    {
		return false;
    }

	Path::iterator it = s->getPath()->end();
	it--;
	return stackMove(s, **(it), true);
}

bool RealPlayer::stackMove(Stack* s, PG_Point dest, bool follow)
{
	debug("player::stack_move()");
	//if follow is set to true, follow an already calculated way, else
	//calculate it here
	if (!follow)
    {
		s->getPath()->calculate(s, dest);
    }
	else
    {
		s->getPath()->checkPath(s);
    }
	
	if (s->getPath()->empty())
    {
		return false;		//way to destination blocked
    }

	while ((enoughMoves(s)) && (s->getPath()->size() > 1))
	{
		stackMoveOneStep(s);
        supdatingStack.emit(0);
	}

	if ((s->getPath()->size() == 1) && enoughMoves(s))
	//now look for fight targets, joins etc.
	{
		PG_Point pos = **(s->getPath()->begin());
		City* city = Citylist::getInstance()->getObjectAt(pos);
		Stack* target = Stacklist::getObjectAt(pos);

		//first fight_city to avoid ambiguity with fight_army
		if (city && (city->getPlayer() != this) && (!city->isBurnt()))
		{
			Fight::RESULT result = stackFightCity
						(vector<Stack*>(1, s), city);

			if (result == Fight::ATTACKER_WON)
			{
				stackMoveOneStep(s);
                invadeCity(city); //determine what to do with city
                supdatingStack.emit(0);
			}
            //if defender won, the signal of a dying stack is already sent
            //elsewhere
			if (result == Fight::NONE)
			{
				s->decrementMoves(1);
					//a fight still costs something
                supdatingStack.emit(0);
			}
			return true;
		}
		//another friendly stack => join it
		else if (target && target->isFriend(this))
		{
			stackMoveOneStep(s);
			stackJoin(s, Stacklist::getAmbiguity(s));
            supdatingStack.emit(0);
			return true;
		}
		//enemy stack => fight
		else if (target)
		{
			Fight::RESULT result = stackFight(s, target);
			if (result == Fight::ATTACKER_WON)
			{
				stackMoveOneStep(s);
                supdatingStack.emit(0);
			}
            //if (result == Fight::DEFENDER_WON)
            //the signal of a dying stack is already sent elsewhere
			if (result == Fight::NONE)
			{
				s->decrementMoves(1);
                supdatingStack.emit(0);
			}
			return true;
		}
		//else
		stackMoveOneStep(s);
        supdatingStack.emit(0);
		return true;
	}

	//If there is another stack where we landed, join it. We can't have two
	//stacks share the same maptile
    if (Stacklist::getAmbiguity(s))
    {
    	stackJoin(s, Stacklist::getAmbiguity(s));
        supdatingStack.emit(0);
    }

	return true;
}

Fight::RESULT RealPlayer::coreStackFight(const char *context, 
                                         vector<Stack*>& attackers,
                                         vector<Stack*>& defenders,
                                         void *client_data)
{
    // We must clean XP to add in case of battle 
    d_defender_xp=0;
    d_attacker_xp=0;

	if (attackers.empty() || defenders.empty())
		return Fight::ATTACKER_WON;

    debug("coreFight: player = " << getName());
    //the timers of bigmap and smallmap cause flickering of the screen, so
    //we remove them for the duration of the fight
    sinterruptTimers.emit();
	
    //first of all, copy the stacks (we need the original ones to compare
    //the wounded and killed units)
    vector<Stack*> orig_defenders, orig_attackers;
    vector<Stack*>::iterator it;
    for (it = defenders.begin(); it != defenders.end(); it++)
    {
        Stack *s = new Stack(**it);
        orig_defenders.push_back(s);
    }
    for (it = attackers.begin(); it != attackers.end(); it++)
    {
        Stack *s = new Stack(**it);
        orig_attackers.push_back(s);
    }
    bool city  = ((context[0] == 'c') && (context[1] == 'i'));
    bool fight = ((context[0] == 'f') && (context[1] == 'i'));
    bool ruin  = ((context[0] == 'r') && (context[1] == 'u'));

    //don't add a defense bonus for ruins
    bool def_bonus = (ruin? false : true);

    //initialize and do the Fight
    //Now it is a pointer because we want to clear the screen before Advancing level
    //if necessary (Andrea)
    FightDialog *fdlg= new FightDialog(attackers, defenders, 0, def_bonus);
    fdlg->Show();
    fdlg->battle();

    Fight::RESULT result = fdlg->getResult();
    delete fdlg;
    //add result to actionlist

    Action *item;

    // fightCity?
    if (city)
    {
	    Action_Fight_City *it = new Action_Fight_City;
	    it->fill_data(orig_attackers, orig_defenders, attackers, defenders);
        item = it;
    }
    // ordinary fight?
    if (fight)
    {
	    Action_Fight *it = new Action_Fight;
	    it->fill_data(attackers[0], defenders[0], 
                      orig_attackers[0], orig_defenders[0]);
        item = it;
    }
    // ruin search?
    if (ruin)
    {
        Action_Search* it = new Action_Search();
        Ruin *r = (Ruin*)client_data;
    	it->fill_data(r, orig_defenders[0], orig_attackers[0], 
                        attackers[0], 
		//searcher didn't kill occupant stack, so ruin is not searched
                        result == Fight::ATTACKER_WON);
        item = it;
    }
    d_actions.push_back(item);

    // clear the copies
    for (it = orig_attackers.begin(); it != orig_attackers.end(); it++)
        Stacklist::deleteStack(*it);
    orig_attackers.clear();
    for (it = orig_defenders.begin(); it != orig_defenders.end(); it++)
        Stacklist::deleteStack(*it);
    orig_defenders.clear();

    culpritsType attackerHeroes, defenderHeroes;

    // get attacker and defender heroes
    getHeroes(attackers, attackerHeroes);
    getHeroes(defenders, defenderHeroes);

    // here we calculate also the total XP to add when a player have a battle
    // clear dead defenders
    debug("clean dead defenders");
    d_defender_xp=removeDeadArmies(defenders, attackerHeroes);

    // and dead attackers
    debug("clean dead attackers");
    d_attacker_xp =removeDeadArmies(attackers, defenderHeroes);

    debug("Attacker totalXP-->"<<  d_attacker_xp <<  "  Defender totalXP-->" <<  d_defender_xp)

    debug("restart timers");
    //don't forget to restart the timers
    scontinueTimers.emit();

    // after a fight: if the attacker's stack won - emit
    // supdatingStack signal, else emit sdyingStack

    debug("after fight: attackers empty? " << attackers.empty()
          << "(" << attackers.size() << ")");


    //only emit this signal when the defender has not won!
    if (attackers.empty() == 0)
    {
        debug("attacker won: supdatingStack");
        supdatingStack.emit(0);
	if (getDefenderXp()!=0) updateArmyXp(attackers,true,getDefenderXp());
    }
    else
    {
        debug("attacker lost: sdyingStack");
        sdyingStack.emit(0);
	if (getAttackerXp()!=0) updateArmyXp(defenders,false,getAttackerXp());
    }

    return result;
}

void RealPlayer::updateArmyXp(vector<Stack*>& stacks,bool emit,double xp_sum)
{
    vector<Stack*>::iterator it;
    
    for (it = stacks.begin(); it != stacks.end(); )
    {
        debug("Stack: " << (*it))
        double numberarmy= (*it)->size();

        for (Stack::iterator sit = (*it)->begin(); sit != (*it)->end();)
        {
            debug("Army: " << (*sit))
            (*sit)->gainXp((double)((xp_sum)/numberarmy));

            //for (int i=0;i<(*sit)->canGainLevel();i++) // Must be <= although you loose a level
            while((*sit)->canGainLevel())  
            {
                //Here this for is to check if army must raise 2 or more levels per time
                //depending on the XP and level itsself

                debug("ADVANCING LEVEL "<< "CANGAINLEVEL== " << (*sit)->canGainLevel())
                if (emit)
                {		
                    //This is the human player army that advance level!
                    std::cerr << "Army Gained Level!" << std::endl;
                    std::cerr << "XP--> " << (*sit)->getXP()  << "  LEVEL== ";
                    std::cerr << (*sit)->getLevel() <<std::endl;
                    debug("Signal gained  Level!")
                    snewlevelArmy.emit(*sit); 
                }
                else 
                {
                    //This is the computer army that advance level!
                    std::cerr << "Army Gained Level!" << std::endl;
                    std::cerr << "XP--> " << (*sit)->getXP()  << "  LEVEL== ";
                    std::cerr << (*sit)->getLevel() <<std::endl;
                    debug("Signal gained  Level!")
                    snewlevelAiArmy.emit(*sit); 
                }	
            }
            debug("Army new XP=" << (*sit)->getXP())
            sit++;
        }	
        it++;
    }
}

// I removed it because now it calculates XP of only dead armies  
/*double RealPlayer::setFighterXp(vector<Stack*>& stacks)
{
    vector<Stack*>::iterator it;
    double total=0;

    for (it = stacks.begin(); it != stacks.end(); )
      {
        for (Stack::iterator sit = (*it)->begin(); sit != (*it)->end();)
	{
            debug("Army: " << (*sit));
	    total+=(*sit)->getXpReward();
	    sit++;
	}	
	it++;
      }

    return total;
}*/

void RealPlayer::getHeroes(vector<Stack*>& stacks, culpritsType& dst)
{
    vector<Stack*>::iterator it;
    for (it = stacks.begin(); it != stacks.end(); it++)
        (*it)->getHeroes(dst);
}

double RealPlayer::removeDeadArmies(vector<Stack*>& stacks,
                                  culpritsType& culprits)
{
    double total=0;
    Player *owner = NULL;
    if (stacks.empty() == 0)
    {
        owner = stacks[0]->getPlayer();
        debug("Owner = " << owner);
        if (owner)
            debug("Owner of the stacks: " << owner->getName()
                  << ", his stacklist = " << owner->getStacklist());
    }
    for (unsigned int i = 0; i < culprits.size(); i++)
        debug("Culprit: " << culprits[i]);

    vector<Stack*>::iterator it;
	for (it = stacks.begin(); it != stacks.end(); )
	{
        std::cerr << "Stack: " << (*it) << "\n";
        for (Stack::iterator sit = (*it)->begin();
				sit != (*it)->end();)
	    {
            std::cerr << "Army: " << (*sit) << "\n";
	       if ((*sit)->getHP() <= 0)
	       {
		 std::cerr << "Army: " << (*sit)->getName() << "\n";
		 std::cerr << "Army: " << (*sit)->getXpReward() << "\n";
		  //Add the XP bonus to the total of the battle;
                  total+=(*sit)->getXpReward();
                  // here we destroy the army, so we send
                  // the signal containing the fight data 
                  std::cerr << "sending sdyingArmy!" << std::endl;
                  sdyingArmy.emit(*sit, culprits);
                  std::cerr << "erasing army!" << std::endl;
		  sit = (*it)->flErase(sit);
                  std::cerr << "army erased" << std::endl;
		  continue;
	       }
	    sit++;
	    }
            std::cerr << "Is stack empty?\n";
	    if ((*it)->empty())
	    {
              if (owner)
              {
                debug("Removing this stack from the owner's stacklist");
                owner->deleteStack(*it);
              } 
              else // there is no owner - like for the ruin's occupants
                debug("No owner for this stack - do stacklist too");

            debug("Removing from the vector too (the vetor had " 
                  << stacks.size() << " elt)");
            it = stacks.erase(it);   
	    } else
                it++;
	}
    debug("after removeDead: size = " << stacks.size());
    return total;
}

Fight::RESULT RealPlayer::stackFight(Stack* sattacker, Stack* sdefender)
{
    if ((sattacker == 0) || (sdefender == 0))
        return Fight::ATTACKER_WON;
    vector<Stack*> defenders(1);
    defenders[0] = sdefender;
    vector<Stack*> attackers(1);
    attackers[0] = sattacker;
    
    debug("stackFight: attacker stack = " << sattacker->getId());
    
    return coreStackFight("fight", attackers, defenders);
}

Fight::RESULT RealPlayer::stackFightCity(vector<Stack*> attackers, 
						City* target)
{
	debug("player::stack_fight_city()")

	vector<Stack*> defenders = Stacklist::defendersInCity(target);
    return coreStackFight("city", attackers, defenders);
}

bool RealPlayer::stackSearchRuin(Stack* s, Ruin* r)
{
	debug("RealPlayer::stack_search_ruin")
	
	//throw out impossible actions
	if ((s->getPos().x != r->getPos().x) ||
		(s->getPos().y != r->getPos().y))
	{
		cerr <<"error, searching stack and ruin to be searched";
		cerr <<"not on same position\n";
		exit(-1);
	}

	if (r->isSearched())
		return false;

	Stack* keeper = r->getOccupant();

    // production variant - there may be defenders
    vector<Stack*> defenders(1);
    defenders[0] = keeper;

    /*
    // variant for testing - no defender
    vector<Stack*> defenders; 
    */

    vector<Stack*> attackers(1);
    attackers[0] = s;

    // This was not used
    //    Fight::RESULT result = coreStackFight("ruin", attackers, defenders, r);   
    coreStackFight("ruin", attackers, defenders, r);

    // special situation - coreStackFight doesn't save history
    // when there were no defenders
    if (keeper == NULL)
    {
        Action_Search* item = new Action_Search();
    	item->fill_data(r, 0, 0, 0, true);
        d_actions.push_back(item);
    }

    // did the explorer lose?
    if (defenders.size())
        return false;

    r->setOccupant(0);
    delete keeper;

    // but if the hero was killed, do not emit the signal about 
    // search
	Army* hero = s->getFirstHero();
    if (hero)
    {
       debug("emiting signal - ruin searched ");
        // emit the signal - (ruin_id, hero_id)
       sruinSearched.emit(r->getId(), hero->getId()); 
    }

    //The fight has been done or left out, now comes the reward. Up to now,
    //we only give some gold (automatically done with give_reward())
    r->setSearched(true);

    // do we reward a player event when his hero died?
    giveReward(1, s);
    supdatingStack.emit(0);
    return true;
}

bool RealPlayer::stackVisitTemple(Stack* s, Temple* t)
{
    debug("RealPlayer::visit_temple")

    //abort in case of impossible action
    if ((t->getPos().x != s->getPos().x)
        || (t->getPos().y != s->getPos().y))
    {
        cerr <<"Stack tried to visit temple at wrong location\n";
        exit(-1);
    }

    //later on, you can also receive quests in temples, but for now it
    //must suffice to have your stack blessed (+1 strength)
    s->bless();

    Action_Temple* item = new Action_Temple();
    item->fill_data(t, s);
    d_actions.push_back(item);

    supdatingStack.emit(0);

    return true;
}

bool RealPlayer::cityOccupy(City* c)
{
    debug("cityOccupy")
    
    c->setPlayer(this);

    //set the production to the cheapest armytype
    c->setProduction(-1);
    if (c->getArmytype(0) != -1)
        c->setProduction(0);

    Action_Occupy* item = new Action_Occupy();
    item->fill_data(c);
    d_actions.push_back(item);

    //to signal that the cities have changed a bit
    supdatingCity.emit(c);

	return true;
}

bool RealPlayer::cityPillage(City* c)
{
    debug("RealPlayer::cityPillage")
    
    int type = -1;
    int no = 0;
    Armyset* as = getArmyset();

    //get the most expensive production in the city
    for (int i = 0; i < 4; i++)
    {
        int temp;
        temp = c->getArmytype(i);
        if (temp == -1)
            continue;
        
        if ((type == -1) || 
            (as->army[temp].getProductionCost() > as->army[type].getProductionCost()))
        {
            type = temp;
            no = i;
        }
    }
    
    if (type == -1)
        return cityOccupy(c);

    addGold(as->army[type].getProductionCost() / 2);
    c->setArmytype(no, -1);

    //assign the city to the player
    c->setPlayer(this);

    //set the production to the cheapest armytype
    c->setProduction(-1);
    if (c->getArmytype(0) != -1)
        c->setProduction(0);

    //insert action item
    Action_Pillage* item = new Action_Pillage();
    item->fill_data(c);
    d_actions.push_back(item);

    supdatingCity.emit(c);

    return true;
}

bool RealPlayer::cityRaze(City* c)
{
    debug("RealPlayer::cityRaze")

	Action_Raze* item = new Action_Raze();
	item->fill_data(c);
	d_actions.push_back(item);

	Citylist::iterator it = Citylist::getInstance()->begin();
	while ((&(*it) != c) && (it != Citylist::getInstance()->end()))
		it++;
	(*it).setBurnt(true);
    //make sure there are no side effects e.g. when counting the cities of
    //a player
    (*it).setPlayer(Playerlist::getInstance()->getNeutral());

    supdatingCity.emit(c);

	return true;
}

bool RealPlayer::cityUpgradeDefense(City* c)
{
	debug("RealPlayer::city_upgrade_defense")

	int needed_gold = c->getDefenseLevel() * 1000;

	if ((needed_gold >= 3000) || (d_gold < needed_gold))
		return false;	//maximum level is 3 or not enough gold
	
	c->setDefenseLevel(c->getDefenseLevel() + 1);
	withdrawGold(needed_gold);

	Action_Upgrade_Defense* item = new Action_Upgrade_Defense();
	item->fill_data(c);
	d_actions.push_back(item);

	return true;
}

bool RealPlayer::cityBuyProduction(City* c, int armytype)
{
	int slot = -1;

	if ((armytype < 0) || (armytype >= getArmyset()->getSize()))
		return false;	//wrong production type (from 0 to 11)

	for (int i = 0; i < 4; i++)
	{
		if ((slot == -1) && (c->getArmytype(i) == -1))
			slot = i;
		if ((c->getArmytype(i) == armytype) &&
            (c->getProductiontype(i).armyset == d_armyset))
			return false;	//city already has this production
	}
	
	if (slot == -1)
		return false;	//city already has all possible production slots
	
	int needed_gold = getArmyset()->army[armytype].getProductionCost();

	if (needed_gold > d_gold)
		return false;	//not enough gold
	
	
	withdrawGold(needed_gold);
	c->setArmytype(slot, armytype);

	Action_Buy_Production* item = new Action_Buy_Production();
	item->fill_data(c, armytype);
	d_actions.push_back(item);

	return true;
}

bool RealPlayer::cityChangeProduction(City* c, int production)
{
	if (production > 4)
		return false;	//cities currently have only 4 slots
	
	if (c->getArmytype(production) == -1)
		return false;	//the demanded production slot doesn't exist
	
	c->setProduction(production);

	Action_Change_Production* item = new Action_Change_Production();
	item->fill_data(c, production);
	d_actions.push_back(item);

	return true;
}

bool RealPlayer::giveReward(int level, Stack* s, fl_Treasure* reward)
{
	debug("RealPlayer::give_reward")

	//by now we don't have the infrastructure to give something different
	//from gold, so this function isn't great

	int gold_amount = rand() % 1000;
	addGold(gold_amount);

	fl_Treasure treasure;
	treasure.type = TREASURE_GOLD;
	treasure.subtype = 0;
	treasure.amount = gold_amount;

	Action_Reward* item = new Action_Reward();
	item->fill_data(0, treasure);
	d_actions.push_back(item);

	return true;
}

bool RealPlayer::stackMoveOneStep(Stack* s)
{
	if (!enoughMoves(s))
		return false;
	
	PG_Point dest = *(s->getPath()->front());
	
	int needed_moves = GameMap::getInstance()->getTile(dest.x,dest.y)->getMoves();
	unsigned int maptype = GameMap::getInstance()->getTile(dest.x,dest.y)->getMaptileType();
	City* c = Citylist::getInstance()->getObjectAt(dest.x, dest.y);

	for (Stack::iterator it = s->begin(); it != s->end(); it++)
	//calculate possible move boni for each army
	{
		if (c != 0)
		//cities cost one MP
		{
			(*it)->decrementMoves(1);
			continue;
		}
		
		if ((*it)->getMoveBonus() == maptype)
		//if army has move bonus, it takes only 2 move points...
		{
			(*it)->decrementMoves(2);
			continue;
		}
		//else the whole
		(*it)->decrementMoves(needed_moves);
	}

	s->moveOneStep();	//this is only for updating positions etc.

	Action_Move_Step* item = new Action_Move_Step();
	item->fill_data(s, dest);
	d_actions.push_back(item);
	
	return true;
}

// End of file
