//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Library General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef W_EDIT_H
#define W_EDIT_H

#include <pgeventobject.h>
#include <pgwidget.h>
#include "GameScenario.h"
#include "NextTurn.h"
#include <sigc++/sigc++.h>

class PG_Label;
class PG_Button;
class BigMap;
class SmallMap;
class Stackinfo;
class NextPlayerPopup;
class Hero;

class W_Edit : public PG_EventObject, public PG_Widget, public SigC::Object
{
	public:
	// CREATORS
    W_Edit(GameScenario* gameScenario, PG_Widget* parent,PG_Rect rect);
    ~W_Edit();

    // CALLBACKS
    PARAGUI_CALLBACK(b_nextClicked);
    PARAGUI_CALLBACK(b_defendClicked);
    PARAGUI_CALLBACK(b_moveClicked);
    PARAGUI_CALLBACK(b_searchClicked);
    PARAGUI_CALLBACK(b_nextTurnClicked);
    PARAGUI_CALLBACK(b_defendAndNextClicked);
    PARAGUI_CALLBACK(b_centerCurrentClicked);

    PARAGUI_CALLBACK(useTurninfo);
    PARAGUI_CALLBACK(stackRedraw);
        
	// MANIPULATORS
    void startGame();
    void loadGame();

    void bigmapRedraw();
    void bigmapStackSelected(Stack* s);
    void cityOccupied(City* city);
    void nextTurn();
    void smallmapRedraw();
    void stackUpdate(Stack* s);
    void stackDied(Stack* s);
    void newLevelArmy(Army* a);
    void lockScreen();
    void unlockScreen();
    void stopTimers();
    void startTimers();
        
    bool eventMouseButtonDown(const SDL_MouseButtonEvent* event);
        
    //a hack to keep GameScenario invisible from MainWindow
    bool save(std::string file){return d_gameScenario->saveGame(file);}

    private:
    void displayFirstCity();
    void pictureNextPlayer();
    bool heroJoins(Hero* hero, int gold);
    void checkButtons();
    void updateStatus();

    bool checkPlayer(Player* p);
    
    // Data
    GameScenario* d_gameScenario;
    NextTurn* d_nextTurn;
    BigMap* d_bigmap;
    SmallMap* d_smallmap;
    Stackinfo* d_stackinfo;
    NextPlayerPopup* d_nextPlayerPopup;
    PG_Label* l_waiting;
    PG_Label* l_turns;
    PG_Label* l_gold;
    SDL_Surface* d_pic_nextTurn;
    SDL_Surface* d_pic_next;
    SDL_Surface* d_pic_defend;
    SDL_Surface* d_pic_move;
    SDL_Surface* d_pic_search;
    SDL_Surface* d_pic_defendAndNext;
    SDL_Surface* d_pic_centerCurrent;

    PG_Button* b_nextTurn;
    PG_Button* b_next;
    PG_Button* b_defend;
    PG_Button* b_move;
    PG_Button* b_search;
    PG_Button* b_defendAndNext;
    PG_Button* b_centerCurrent;
    
    bool d_allDefending;
    bool d_lock;
};

#endif // W_EDIT_H

// End of file
